/*
 * Copyright (C) 2011-2012 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

`define SUPPORT_PG		
`define SUPPORT_OVR		

//
// Module Name: alu381
//   ver 001
// 
//         original 74381       (382)
//    A1  1            24  VCC  
//    B1  2            23  A2 
//    A0  3            22  B2 
//    B0  4            21  A3  
//    S0  5            20  B3 
//    S1  6            19  Cn  
//    S2  7            18  ~P  Cn+4
//    F0  8            17  ~G  OVR
//    F1  9            16  F3
//   GND  12           13  F2 
//

module alu381 (
		 input   C_IN        // Carry Input
		, input  [3:0] A     // DATA IN
		, input  [3:0] B     // DATA IN
		, output [3:0] F     // DATA OUT
`ifdef SUPPORT_PG
		, output  P          // carry-propagate (INV)
		, output  G          // carry-generate (INV)
`endif
`ifdef SUPPORT_OVR
		, output  C_OUT      // carry-look-ahead
		, output  OVR        // overflow
`endif
		, input   [2:0] S   // Function Select
		);

	wire cin;
	wire M;
	wire SWAP;
	wire [7:0] FS;
	alu4 alu_impl ( .A(A[3:0]), .B(B[3:0]), .F(F[3:0]), .S(FS[7:0])
			, .M(M) , .C_IN(cin) 
`ifdef SUPPORT_PG
			, .P(P) , .G(G)
`endif
`ifdef SUPPORT_OVR
			, .OVR(OVR) , .C_OUT(cout)
`endif
			);
	assign M = (S == 0) | (S == 4) | (S == 5) | ( S == 6);
	assign FS = (S == 0) ? 8'b10010011 // CLEAR
		   :(S == 1) ? 8'b01101001 // B - A
		   :(S == 2) ? 8'b10010110 // A - B
		   :(S == 3) ? 8'b10011001 // A + B
		   :(S == 4) ? 8'b10010110 // A ^ B
		   :(S == 5) ? 8'b10011110 // A | B
		   :(S == 6) ? 8'b10011011 // A & B
		   :           8'b10010011 // PRESET
		   ;
	assign cin = ~( S == 7) & C_IN;
`ifdef SUPPORT_OVR
	assign C_OUT = ~cout;
`endif
endmodule
