/*
 * Copyright (C) 2011 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
module dac3 # (
		parameter WIDTH = 16
		, parameter WITH_LATCH = 1
		, parameter MODE = 1	// Phase Correct Mode
	) (
		input CLK
		,input STB // active high
                ,input [WIDTH-1:0] I_DATA
		,output [2:0] A_OUT
		,output O_STB
	);

    reg r_stb = 1'b0;
    reg [WIDTH-1:0] r_data = 0;
    reg [2:0] r_data2 = 0;
    reg [WIDTH-3:0] count = 0;
    reg r_out0 = 1'b0;
    reg [2:0] r_out = 0;
    
    wire [WIDTH-3-1:0] WK = 0;
    wire [WIDTH-1:0] I_MAX = { 3'b110 , ~WK }; 
    wire [WIDTH-1:0] I_DATA2 = (&I_DATA[WIDTH-1:WIDTH-3])
		 		? I_DATA : I_MAX;

    assign A_OUT = r_out;
    wire O_STB = &count;

    always @(posedge CLK)
    begin
	r_stb <= STB;
	if (&count | (STB & ~r_stb))
	  begin
		count <= 0;
		if (WITH_LATCH)
		  begin
			r_data <= I_DATA2;
			r_data2 <= I_DATA2[WIDTH-1:WIDTH-3] + 1;
		  end
	  end
	else	count <= count + 1;

	r_out0 <= (((MODE & count[WIDTH-3])? ~count[WIDTH-3-1:0]: count[WIDTH-3-1:0])
			 < (WITH_LATCH ? r_data : I_DATA) );
	r_out <= r_out0 ? (WITH_LATCH ? r_data2 : I_DATA2[WIDTH-1:WIDTH-3] + 1)
		   : (WITH_LATCH ? r_data[WIDTH-1:WIDTH-3]: I_DATA2[WIDTH-1:WIDTH-3]);
    end
endmodule
