/*
 * Copyright (C) 2011 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
module dac # (
		parameter WIDTH = 16
		, parameter WITH_LATCH = 1
		, parameter MODE = 3
	) (
		input CLK
		,input STB // active high
                ,input [WIDTH-1:0] I_DATA
		,output A_OUT
		,output O_STB
	);

    reg r_stb = 1'b0;
    reg [WIDTH-1:0] r_data = 0;
    reg [WIDTH-1:0] fraction = 0;
    reg [WIDTH-1:0] count = 0;
    reg r_out = 1'b0;
    reg [MODE:0] r_pc = 0;

    assign A_OUT = MODE ? r_pc[MODE] : r_out;
    wire O_STB = &count;

    always @(posedge CLK)
    begin
	r_stb <= STB;
	if (&count | (STB & ~r_stb))
	  begin
		count <= 0;
		if (WITH_LATCH) r_data <= I_DATA;
	  end
	else 	count <= count + 1;

	{ r_out, fraction } <= { 1'b0 ,fraction }
			 + { 1'b0, (WITH_LATCH ? r_data : I_DATA) };

	if (MODE == 2)
	    r_pc <= ((r_pc == 3'b010) | (r_pc == 3'b101)) ?
		   { ~r_pc[1], ~r_pc[0] , r_out }
		:  { r_pc[1], r_pc[0] , r_out };
	else if (MODE == 3)
	    r_pc <= ((r_pc[3:1] == 3'b010) | (r_pc[3:1] == 3'b101)) ?
			{ ~r_pc[2], ~r_pc[1] , r_pc[0], r_out }
	        : ((r_pc == 4'b0110) | (r_pc == 4'b1001)) ?
			{ ~r_pc[2],  r_pc[1], ~r_pc[0] , r_out }
		:	{  r_pc[2],  r_pc[1],  r_pc[0] , r_out };
    end
endmodule
