/*
 * Copyright (C) 2011 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* Sigma Delta DAC using 5bit R-2R DAC */
module dac5 # (
		parameter AWIDTH = 4
		, parameter WIDTH = 16
		, parameter WITH_LATCH = 1
	) (
		input CLK
		,input STB // active high
                ,input [WIDTH-1:0] I_DATA
		,output [AWIDTH-1:0] A_OUT
		,output O_STB
	);

    reg r_stb = 1'b0;
    reg [WIDTH-1:0] r_data = 0;
    reg [WIDTH-AWIDTH-1:0] fraction = 0;
    reg [WIDTH-AWIDTH-1:0] count = 0;
    reg [AWIDTH-1:0] r_out = 0;

    wire [WIDTH-AWIDTH-1:0] WK = 0;
    wire [AWIDTH-1:0] A_ZERO = 0;
    wire [AWIDTH-1:0] A_WK = ~A_ZERO -1;
    wire [WIDTH-1:0] I_MAX = { A_WK , ~WK }; 
    wire [WIDTH-1:0] I_DATA2 = (&I_DATA[WIDTH-1:WIDTH-AWIDTH])
		 		? I_DATA : I_MAX;

    assign A_OUT[AWIDTH-1:0] = r_out[AWIDTH-1:0];
    wire O_STB = &count;

    always @(posedge CLK)
    begin
	r_stb <= STB;
	if (&count | (STB & ~r_stb))
	  begin
		count <= 0;
		if (WITH_LATCH) r_data <= I_DATA2;
	  end
	else	count <= count + 1;

	{ r_out, fraction } <= { A_ZERO , fraction }
				 + (WITH_LATCH ? r_data : I_DATA2 );
    end
endmodule
