//`define EXTOSC_PASSIVE

`define JTAG_BSCAN JTAGF

module tool_test (
		// JTAG 
		  input TOP_TDI
		, input TOP_TCK
		, input TOP_TMS
		, output  TOP_TDO

		, inout SDA
		, inout SCL

		// External OSC
		, input EXTOSC
		, output EXTOSC_EN
		);

PUR pur(.PUR(1'b1));

  `JTAG_BSCAN bscan (
	.JRSTN(BSCAN_RESET) // H : in TestLogicReset State

	, .JSHIFT(BSCAN_SHIFT)

	, .TDI(TOP_TDI)
	, .TCK(TOP_TCK)
	, .TMS(TOP_TMS) 
	, .TDO(TOP_TDO)

	, .JTDI(JTDI_IN)
	, .JTCK(JTCK_IN) // negedge capt, posedge shift

	// signals for IPA
	, .JRTI1(JRTI1)
	, .JCE1(SEL1)
	, .JTDO1(JTDO1)

	// signals for IPB
	, .JRTI2(JRTI2)
	, .JCE2(SEL2)
	, .JTDO2(JTDO2) 
	);

    wire BSCAN_RESET_N = ~BSCAN_RESET;

  wire CLK;
  wire RESET;
//############  SPI/ISP TIMING GENERATION #####################################

  reg r_jtck;
  reg r_jtck_prev;
  reg r_cs1;
  reg r_cs1_prev;
  reg r_cs2;
  reg r_cs2_prev;
  
  always @(posedge CLK)
    begin
	r_jtck_prev <= JTCK_IN;
	if (r_jtck_prev == JTCK_IN)
		r_jtck <= ~r_jtck_prev;
	if (r_jtck_prev & r_jtck & (r_jtck_prev == JTCK_IN)) // negedge JTCK
	    begin
		    r_cs1_prev <= ~(SEL1 & BSCAN_SHIFT);
		    r_cs1      <= r_cs1_prev;
		    r_cs2_prev <= ~(SEL2 & BSCAN_SHIFT);
		    r_cs2      <= r_cs2_prev;
	    end
    end
  wire JTCK1 = r_jtck;
  wire JTCK2 = r_jtck;
  wire JTDI  = JTDI_IN;
  wire CS1 = r_cs1;
  wire CS2 = r_cs2;

//############  SPI TEST MODULE #####################################

  wire [7:0] SPI_DATA;
  wire [7:0] SPI_DATA_I;
  wire [1:0] SPI_COUNT;
  reg [7:0] r_data;

  wire SPI_DATA_ACK;
  jtag_spi JTAG_SPI (.CLK(CLK)
		, .MOSI(JTDI)
                , .SCK(JTCK1)
                , .CS(CS1)
                , .MISO(JTDO1)
		, .SPI_DATA(SPI_DATA)
		, .SPI_DATA_VALID(SPI_DATA_VALID)
		, .SPI_DATA_REQ(SPI_DATA_REQ)
		, .SPI_DATA_ACK(SPI_DATA_ACK)
		, .SPI_DATA_COUNT(SPI_COUNT)
		, .SPI_DATA_I(SPI_DATA_I)
		);

`ifdef XXX
   assign SPI_DATA_ACK = 1'b1;
   always @(posedge CLK)
      begin
	if (SPI_DATA_VALID) 
		r_data <= SPI_DATA;
      end
    assign SPI_DATA_I[7:0] = r_data[7:0]; // latched version
    //assign SPI_DATA_I[7:0] = SPI_DATA[7:0]; // path thorugh 

`else
    reg WE;
    reg [1:0] wb_stat = 1'b0;
    reg wb_cyc = 1'b0;
    reg wb_stb = 1'b0;
    reg [7:0] wb_adr;
    wire [7:0] wb_data;
    wire RST_NEG = RESET;
    reg r_data_ack = 1'b0;
    assign SPI_DATA_ACK = r_data_ack;

    EFB_tc_i2c efb_impl (.wb_clk_i(CLK), .wb_rst_i(RESET)
    		  , .wb_adr_i(wb_adr), .wb_dat_i(SPI_DATA), .wb_dat_o(wb_data)
		  , .wb_we_i(WE), .wb_cyc_i(wb_cyc), .wb_stb_i(wb_stb)
		  , .tc_clki(CLK), .tc_rstn(RST_NEG)
		  , .i2c1_scl(SCL), .i2c1_sda(SDA)
		);

    always @(posedge CLK)
    begin
	if (wb_stat == 1)
	  begin
			wb_stat <= 2;
			r_data_ack <= 1'b0;
	  end
	else if (wb_stat == 2)
	  begin
			wb_stat <= 3;
			wb_cyc <= 1'b0;
			wb_stb <= 1'b0;
			WE <= 1'b0;
	  end
	else if ((SPI_COUNT[0] == 1'b0) && SPI_DATA_REQ)
	  begin
			wb_stat <= 1;
			wb_cyc <= 1'b1;
			wb_stb <= 1'b1;
			wb_adr[7:0] <= { 1'b0, SPI_DATA[6:0] };
			r_data_ack <= 1'b0;
	  end
	else if ((SPI_COUNT[1] == 1'b1) & SPI_DATA_VALID)
	  begin
			WE <= ~SPI_DATA[0];
	  end
	else if ((SPI_COUNT[1] == 1'b1) & SPI_DATA_REQ)
	  begin
			r_data_ack <= 1'b1;
	  end
	else if ((SPI_COUNT[0] == 1'b1) & SPI_DATA_VALID & WE)
	  begin
			wb_stat <= 1;
			wb_cyc <= 1'b1;
			wb_stb <= 1'b1;
	  end
    end
`endif

//############   #####################################

  //OSCH #(.NOM_FREQ("88.67")) osc_internal
  //OSCH #(.NOM_FREQ("44.33")) osc_internal
    OSCH #(.NOM_FREQ("24.18")) osc_internal
  //OSCH #(.NOM_FREQ("10.23")) osc_internal // 10.23 MHz 
	(.OSC(CLK_INT), .STDBY(1'b0));

 `ifdef EXTOSC_PASSIVE
    wire EXTOSC_EN = ~EXTOSC;
 `else
    wire EXTOSC_EN = 1'b1;
  `endif

  DCMA clk_selector (.CLK0(CLK_INT)
        , .CLK1(EXTOSC)
        , .SEL(`CLK_SEL) // 0: CLK0 1: CLK1
        , .DCMOUT(CLK_IN0)
        );

`define USE_RESET_COUNTER
`ifdef USE_RESET_COUNTER
  reg [3:0] reset_cnt = 0;
  wire r_reset = (reset_cnt != 4'b1111);
  always @(posedge CLK_IN0)
    begin
        if (reset_cnt != 4'b1111)
            reset_cnt <=  reset_cnt + 1;
    end
  assign RESET = r_reset;
`else
  reg [15:0] reset_sft     = 16'h0000;
    always @(posedge CLK_IN0)
    begin
        reset_sft <=  { reset_sft[15-1:0] , 1'b1 };
    end
  assign RESET = reset_sft[15];
`endif

  assign CLK_IN = CLK_IN0;
  assign CLK    = CLK_IN;

endmodule
